| print.test.nleqslv | R Documentation |
testnslvPrint method for test.nleqslv objects.
## S3 method for class 'test.nleqslv'
print(x, digits = 4, width.cutoff = 45L, ...)
x |
a |
digits |
specifies the minimum number of significant digits to be printed in values. |
width.cutoff |
integer passed to |
... |
additional arguments to |
This is the print method for objects inheriting from class
test.nleqslv. It prints the call to testnslv followed by the
description of the experiment (if the title argument was specified in
the call to testnslv) and the dataframe containing the results of
testnslv.
It returns the object x invisibly.
dslnex <- function(x) {
y <- numeric(2)
y[1] <- x[1]^2 + x[2]^2 - 2
y[2] <- exp(x[1]-1) + x[2]^3 - 2
y
}
xstart <- c(1.5,0.5)
fstart <- dslnex(xstart)
z <- testnslv(xstart,dslnex)
print(z)
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.